# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .phone_number_response import PhoneNumberResponse

__all__ = ["PhoneNumberListResponse"]

PhoneNumberListResponse: TypeAlias = List[PhoneNumberResponse]
